/* -*- mode: c++; -*-
 *-----------------------------------------------------------------------------
 * $RCSfile: XmlModifier.cpp,v $
 *
 * See Copyright for the status of this software.
 *
 * The OpenSOAP Project
 * http://opensoap.jp/
 *-----------------------------------------------------------------------------
 */

#include <iostream>
#include <string>

#include <stdexcept>

#ifdef HAVE_CONFIG_H
# include "config.h"
#endif /* HAVE_CONFIG_H */

/* libxml2 */
#if defined(HAVE_LIBXML2_XMLVERSION_H) 
#include <libxml2/tree.h>
#include <libxml2/parser.h>
#else
#include <libxml/tree.h>
#include <libxml/parser.h>
#endif /* !HAVE_LIBXML2_XMLVERSION_H */

#include "ServerCommon.h"
#include "XmlModifier.h"

#include "XmlQuery2.h"

using namespace OpenSOAP;
using namespace std;

// for LOG
static const std::string CLASS_SIG = "XmlModifier";

XmlModifier::XmlModifier(const std::string& xmlMsg)
  : xmlDoc(NULL)
{
  /* COMPAT: Do not generate nodes for formatting spaces */

  LIBXML_TEST_VERSION xmlKeepBlanksDefault(0);

  char* xmlMsgChar = new char[xmlMsg.length()+1];
  memcpy (xmlMsgChar, xmlMsg.c_str(), xmlMsg.length());
  xmlMsgChar[xmlMsg.length()] = 0x00;

  xmlDoc = xmlParseMemory(xmlMsgChar, xmlMsg.length());
  //xmlDoc = xmlParseMemory((char*)xmlMsg.c_str(), xmlMsg.length());
  if (NULL == xmlDoc) {
    cerr << TAG_ERR << CLASS_SIG << "::" << CLASS_SIG
	 << ": xmlParseMemory returned NULL" << endl;
	delete[] xmlMsgChar;
    throw std::exception();
  }
  /* check document (root node) */
  xmlNodePtr node1 = xmlDocGetRootElement(xmlDoc);
  if (node1 == NULL) {
    cerr << TAG_ERR << CLASS_SIG << "::" << CLASS_SIG
	 << ": empty document" << endl;
    xmlFreeDoc(xmlDoc);
	xmlDoc = NULL;
	delete[] xmlMsgChar;
    throw std::exception();
  }

	delete[] xmlMsgChar;
}

XmlModifier::~XmlModifier()
{
  if (xmlDoc) {
    xmlFreeDoc(xmlDoc);
  }
}

std::string 
XmlModifier::toString()
{
  /* output doc tree */
  xmlChar* memPtr;
  int size;
  xmlDocDumpMemory(xmlDoc, &memPtr, &size);
  
  return std::string((char*)memPtr, size);
}

int
XmlModifier::attach(const std::string& fmt, const std::string& value)
{
  XmlQuery2 xmlQuery(fmt);
  return xmlQuery.attach(xmlDoc, xmlDocGetRootElement(xmlDoc), NULL,
			 value,
			 false, false, NULL);
}

int
XmlModifier::attachNoDuplicate(const std::string& fmt, 
			       const std::string& value)
{
  XmlQuery2 xmlQuery(fmt);
  std::vector<std::string> tmpBuf;
  int ret = xmlQuery.getValue(xmlDoc, xmlDocGetRootElement(xmlDoc), tmpBuf);
  if (ret == 0) {
    return 0;
  }
  return attach(fmt, value);
}

int 
XmlModifier::attach(const std::string& fmt, 
		    const std::string& value,
		    const NameSpaceDef* nsDef)
{
  XmlQuery2 xmlQuery(fmt);
  XmlQuery2::NameSpaceDef* def = NULL;
  if (nsDef) {
    def = new XmlQuery2::NameSpaceDef;
    def->href = nsDef->href;
    def->prefix = nsDef->prefix;
  }
  //edit 2003/02/06 memory leak
  int ret = xmlQuery.attach(xmlDoc, xmlDocGetRootElement(xmlDoc), NULL,
			    value,
			    false, true, def);
  if (def) {
    delete def;
  }
  return ret;
}

int 
XmlModifier::attachNoDuplicate(const std::string& fmt, 
		    const std::string& value,
		    const NameSpaceDef* nsDef)
{
  XmlQuery2 xmlQuery(fmt);
  std::vector<std::string> tmpBuf;
  int ret = xmlQuery.getValue(xmlDoc, xmlDocGetRootElement(xmlDoc), tmpBuf);
#ifdef DEBUG
  cerr << "DupChk=(" << ret << ")" << endl;
#endif
  if (ret == 0) {
    return 0;
  }
  return attach(fmt, value, nsDef);
}

int 
XmlModifier::update(const std::string& fmt, const std::string& value)
{
  XmlQuery2 xmlQuery(fmt);
  return xmlQuery.attach(xmlDoc, xmlDocGetRootElement(xmlDoc), NULL,
			 value,
			 true, false, NULL);
}

int
XmlModifier::append(const std::string& fmt, const std::string& value)
{
  XmlQuery2 xmlQuery(fmt);
  return xmlQuery.attach(xmlDoc, xmlDocGetRootElement(xmlDoc), NULL,
			 value,
			 false, false, NULL, 1);
}


int 
XmlModifier::append(const std::string& fmt, 
		    const std::string& value,
		    const NameSpaceDef* nsDef)
{
  XmlQuery2 xmlQuery(fmt);
  XmlQuery2::NameSpaceDef* def = NULL;
  if (nsDef) {
    def = new XmlQuery2::NameSpaceDef;
    def->href = nsDef->href;
    def->prefix = nsDef->prefix;
  }
  //edit 2003/02/06 memory leak
  int ret = xmlQuery.attach(xmlDoc, xmlDocGetRootElement(xmlDoc), NULL,
			    value,
			    false, true, def, 1);
  if (def) {
    delete def;
  }
  return ret;
}

int 
XmlModifier::del(const std::string& fmt)
{
  XmlQuery2 xmlQuery(fmt);
  return xmlQuery.del(xmlDoc, xmlDocGetRootElement(xmlDoc));
}


    
